function [phi, resid, rssq] =  setar4estim(y, thres, ttau, d, p)
%    TAR (time) + TAR (state) program parameter estimate
%    y= time series data, column vector
%    thres =  threshold values, including interval extremes
%    ttau  =  time breaks, first element 0 last element n
%    d  =  delay parameter
%    p  =  autoregressive order
% --- notice thres: thres(1)=value less than the smallest observation, 
%       thres(nr+1)=largest observation (the program check for <=)
%    OUTPUT parameter phi each row is an AR parameter sequence, rows are
%    ordered state regime (sequence of time break AR for each state regime)
%    RESIDUALS in output are a column vector
%    rssq  is the RESIDUAL SUM OF SQUARES, not the residual variance
n = size(y, 1);
nr1 = size(thres, 2); nr = nr1-1; %   nr = number of regimes in state levels
nt1 =  size(ttau,2); nt = nt1-1;  %  nt = number of regimes in time
m =  max(p, d); % observations lost at the beginning of the time series
Z = y(m+1:n); % dependent variable
ZS = y(m-d+1:n-d); %  state variable
ti=(1:1:n-m)';  %  time
  %   size(ZS), size(ti),
ZMAT = ones(n-m, p+1); % initialize the indpendent variables
phi =  zeros(nr*nt, p+1);  %  initialize the parameter array
for ic = 1:p, ZMAT(:,ic+1) = y(m-ic+1:n-ic); end, %ZMAT, % compute the independet var
    yp = 0; % parameter row: 1=1,1; 2=1,2; 3=2,1; 4=2,2;
for  regime = 1:nr, for tau=1:nt, % for each regime performs least squares estimation
    clear r1,  clear r2, r1 = thres(regime);  r2 = thres(regime+1);  %  r2=threshold
    clear t1,  clear t2, t1 = ttau(tau);  t2 = ttau(tau+1);  %  t2=time break
 %   idx  is the array that includes the indexes of the observations in the (cross)regime
    clear idx, idx  =  find((ZS>r1 & ZS<=r2)&(ti>t1-m & ti<=t2-m)); % size(idx),
    clear X, clear XMAT, X = Z(idx); XMAT= ZMAT(idx,:); %  prepare for regression
       yp = yp + 1;
    phi(yp,:) = (XMAT\X)';  % lesast squraes solution of equation XMAT*phi=X
end, end,
%phi, 
resid = zeros(n,1); % initialize the residual column vector
    yp = 0; % parameter row: 1=1,1; 2=1,2; 3=2,1; 4=2,2;
for  regime = 1:nr, for tau=1:nt,  % for each regime compute the residuals
    clear r1,  clear r2, r1 = thres(regime);  r2 = thres(regime+1);  %  r2=threshold
    clear t1,  clear t2, t1 = ttau(tau);  t2 = ttau(tau+1);  %  t2=time break
 %   idx  is the array that includes the indexes of the observations in the (cross)regime
    clear idx, idx  =  find((ZS>r1 & ZS<=r2)&(ti>t1-m & ti<=t2-m)); 
    clear X, clear XMAT, X = Z(idx); XMAT= ZMAT(idx,:); %  prepare for regression
       yp = yp + 1;
    clear th, th=(phi(yp,:))'; resid(idx+m)=X-XMAT*th;  % residuals in regime with y*=XMAT*th
end, end,
%rssq = norm(resid), %  norm computes the square root of the sum of squares 
rssq  =  resid'*resid;   %  the residual sum of squares 
%   setar4estim  program estimate ends